@@ -1,4 +1,5 @@
-# Copyright 2014 eNovance
+#
+# Copyright 2012 Intel Corp.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -11,7 +12,8 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+#
 
+import gettext
 
-class NotImplementedError(NotImplementedError):
-    pass
+gettext.install('ceilometer', unicode=1)
