@@ -1,5 +1,8 @@
+# -*- encoding: utf-8 -*-
 #
-# Copyright 2012 New Dream Network, LLC (DreamHost)
+# Copyright © 2012 New Dream Network, LLC (DreamHost)
+#
+# Author: Doug Hellmann <doug.hellmann@dreamhost.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -15,17 +18,34 @@
 """Simple logging storage backend.
 """
 
-from oslo_log import log
-
+from ceilometer.openstack.common import log
 from ceilometer.storage import base
 
 LOG = log.getLogger(__name__)
 
 
-class Connection(base.Connection):
-    """Log the data."""
+class LogStorage(base.StorageEngine):
+    """Log the data
+    """
 
-    def upgrade(self):
+    def register_opts(self, conf):
+        """Register any configuration options used by this engine.
+        """
+
+    def get_connection(self, conf):
+        """Return a Connection instance based on the configuration settings.
+        """
+        return Connection(conf)
+
+
+class Connection(base.Connection):
+    """Base class for storage system connections.
+    """
+
+    def __init__(self, conf):
+        return
+
+    def upgrade(self, version=None):
         pass
 
     def clear(self):
@@ -35,26 +55,30 @@ class Connection(base.Connection):
         """Write the data to the backend storage system.
 
         :param data: a dictionary such as returned by
-                     ceilometer.meter.meter_message_from_counter.
+                     ceilometer.meter.meter_message_from_counter
         """
-        LOG.info('metering data %(counter_name)s for %(resource_id)s: '
-                 '%(counter_volume)s'
-                 % ({'counter_name': data['counter_name'],
-                     'resource_id': data['resource_id'],
-                     'counter_volume': data['counter_volume']}))
+        LOG.info('metering data %s for %s: %s',
+                 data['counter_name'],
+                 data['resource_id'],
+                 data['counter_volume'])
 
-    def clear_expired_metering_data(self, ttl):
-        """Clear expired data from the backend storage system.
+    def get_users(self, source=None):
+        """Return an iterable of user id strings.
 
-        Clearing occurs according to the time-to-live.
-        :param ttl: Number of seconds to keep records for.
+        :param source: Optional source filter.
         """
-        LOG.info("Dropping metering data with TTL %d", ttl)
+        return []
+
+    def get_projects(self, source=None):
+        """Return an iterable of project id strings.
+
+        :param source: Optional source filter.
+        """
+        return []
 
     def get_resources(self, user=None, project=None, source=None,
-                      start_timestamp=None, start_timestamp_op=None,
-                      end_timestamp=None, end_timestamp_op=None,
-                      metaquery=None, resource=None, limit=None):
+                      start_timestamp=None, end_timestamp=None,
+                      metaquery={}, resource=None):
         """Return an iterable of dictionaries containing resource information.
 
         { 'resource_id': UUID of the resource,
@@ -69,21 +93,18 @@ class Connection(base.Connection):
         :param project: Optional ID for project that owns the resource.
         :param source: Optional source filter.
         :param start_timestamp: Optional modified timestamp start range.
-        :param start_timestamp_op: Optional start time operator, like gt, ge.
         :param end_timestamp: Optional modified timestamp end range.
-        :param end_timestamp_op: Optional end time operator, like lt, le.
         :param metaquery: Optional dict with metadata to match on.
         :param resource: Optional resource filter.
-        :param limit: Maximum number of results to return.
         """
         return []
 
     def get_meters(self, user=None, project=None, resource=None, source=None,
-                   limit=None, metaquery=None, unique=False):
+                   metaquery={}):
         """Return an iterable of dictionaries containing meter information.
 
         { 'name': name of the meter,
-          'type': type of the meter (gauge, delta, cumulative),
+          'type': type of the meter (guage, counter),
           'resource_id': UUID of the resource,
           'project_id': UUID of project owning the resource,
           'user_id': UUID of user owning the resource,
@@ -93,25 +114,35 @@ class Connection(base.Connection):
         :param project: Optional ID for project that owns the resource.
         :param resource: Optional resource filter.
         :param source: Optional source filter.
-        :param limit: Maximum number of results to return.
         :param metaquery: Optional dict with metadata to match on.
-        :param unique: If set to true, return only unique meter information.
         """
         return []
 
-    def get_samples(self, sample_filter, limit=None):
-        """Return an iterable of samples.
-
-        Items are created by
-        ceilometer.publisher.utils.meter_message_from_counter.
+    def get_samples(self, event_filter):
+        """Return an iterable of samples as created by
+        :func:`ceilometer.meter.meter_message_from_counter`.
         """
         return []
 
-    def get_meter_statistics(self, sample_filter, period=None, groupby=None,
-                             aggregate=None):
+    def get_volume_sum(self, event_filter):
+        """Return the sum of the volume field for the samples
+        described by the query parameters.
+        """
+
+    def get_volume_max(self, event_filter):
+        """Return the maximum of the volume field for the samples
+        described by the query parameters.
+        """
+
+    def get_event_interval(self, event_filter):
+        """Return the min and max timestamp for samples
+        matching the event_filter.
+        """
+
+    def get_meter_statistics(self, event_filter, period=None):
         """Return a dictionary containing meter statistics.
+        described by the query parameters.
 
-        Meter statistics is described by the query parameters.
         The filter must have a meter value set.
 
         { 'min':
@@ -126,5 +157,5 @@ class Connection(base.Connection):
           'duration_start':
           'duration_end':
           }
+
         """
-        return []
