@@ -1,3 +1,7 @@
+# -*- encoding: utf-8 -*-
+#
+# Author: Julien Danjou <julien@danjou.info>
+#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -10,14 +14,20 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from sqlalchemy import Column
-from sqlalchemy import Integer
-from sqlalchemy import MetaData
-from sqlalchemy import Table
+from sqlalchemy import *
+
+meta = MetaData()
 
 
 def upgrade(migrate_engine):
-    meta = MetaData(bind=migrate_engine)
+    meta.bind = migrate_engine
     meter = Table('meter', meta, autoload=True)
     duration = Column('counter_duration', Integer)
     meter.drop_column(duration)
+
+
+def downgrade(migrate_engine):
+    meta.bind = migrate_engine
+    meter = Table('meter', meta, autoload=True)
+    duration = Column('counter_duration', Integer)
+    meter.create_column(duration)
