@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2012 Canonical.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,12 +20,25 @@ def upgrade(migrate_engine):
     if migrate_engine.name == "mysql":
         tables = ['meter', 'user', 'resource', 'project', 'source',
                   'sourceassoc']
-        migrate_engine.execute("SET foreign_key_checks = 0")
+        sql = "SET foreign_key_checks = 0;"
 
         for table in tables:
-            migrate_engine.execute(
-                "ALTER TABLE %s CONVERT TO CHARACTER SET utf8" % table)
-        migrate_engine.execute("SET foreign_key_checks = 1")
-        migrate_engine.execute(
-            "ALTER DATABASE %s DEFAULT CHARACTER SET utf8" %
-            migrate_engine.url.database)
+            sql += "ALTER TABLE %s CONVERT TO CHARACTER SET utf8;" % table
+        sql += "SET foreign_key_checks = 1;"
+        sql += "ALTER DATABASE %s DEFAULT CHARACTER SET utf8;" \
+            % migrate_engine.url.database
+        migrate_engine.execute(sql)
+
+
+def downgrade(migrate_engine):
+    # Operations to reverse the above upgrade go here.
+    if migrate_engine.name == "mysql":
+        tables = ['meter', 'user', 'resource', 'project', 'source',
+                  'sourceassoc']
+        sql = "SET foreign_key_checks = 0;"
+
+        for table in tables:
+            sql += "ALTER TABLE %s CONVERT TO CHARACTER SET latin1;" % table
+        sql += "SET foreign_key_checks = 1;"
+        sql += "ALTER DATABASE %s DEFAULT CHARACTER SET latin1;" \
+            % migrate_engine.url.database
