@@ -1,5 +1,8 @@
+# -*- encoding: utf-8 -*-
 #
-# Copyright 2013 eNovance
+# Copyright © 2013 eNovance
+#
+# Author: Julien Danjou <julien@danjou.info>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -12,13 +15,26 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
 from sqlalchemy import MetaData, Table, Column, DateTime
 
+meta = MetaData()
+
 
 def upgrade(migrate_engine):
-    meta = MetaData(bind=migrate_engine)
+    meta.bind = migrate_engine
     resource = Table('resource', meta, autoload=True)
     timestamp = Column('timestamp', DateTime)
     resource.drop_column(timestamp)
     received_timestamp = Column('received_timestamp', DateTime)
     resource.drop_column(received_timestamp)
+
+
+def downgrade(migrate_engine):
+    meta.bind = migrate_engine
+    resource = Table('resource', meta, autoload=True)
+    timestamp = Column('timestamp', DateTime)
+    resource.create_column(timestamp)
+    received_timestamp = Column('received_timestamp', DateTime,
+                                default=timeutils.utcnow)
+    resource.create_column(received_timestamp)
