@@ -1,6 +1,7 @@
-# -*- coding: utf-8 -*-
+# -*- encoding: utf-8 -*-
 #
-# Copyright 2013 eNovance SAS <licensing@enovance.com>
+# Copyright © 2013 eNovance SAS <licensing@enovance.com>
+# Author: François Charlier <francois.charlier@enovance.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -14,12 +15,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from sqlalchemy import Float
-from sqlalchemy import MetaData
-from sqlalchemy import Table
+from sqlalchemy import *
+
+meta = MetaData()
 
 
 def upgrade(migrate_engine):
-    meta = MetaData(bind=migrate_engine)
+    meta.bind = migrate_engine
     meter = Table('meter', meta, autoload=True)
     meter.c.counter_volume.alter(type=Float(53))
+
+
+def downgrade(migrate_engine):
+    meta.bind = migrate_engine
+    meter = Table('meter', meta, autoload=True)
+    meter.c.counter_volume.alter(type=Integer)
