@@ -1,5 +1,8 @@
+# -*- encoding: utf-8 -*-
 #
-# Copyright 2013 Julien Danjou
+# Copyright © 2013 Julien Danjou
+#
+# Author: Julien Danjou <julien@danjou.info>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -13,32 +16,28 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from ceilometer import transformer
+from ceilometer import plugin
 
 
-class TransformerAccumulator(transformer.TransformerBase):
-    """Transformer that accumulates samples until a threshold.
-
-    And then flushes them out into the wild.
-    """
-
-    grouping_keys = ['resource_id']
+class TransformerAccumulator(plugin.TransformerBase):
+    """Transformer that accumulates counter until a threshold, and then flush
+    them out in the wild. """
 
     def __init__(self, size=1, **kwargs):
         if size >= 1:
-            self.samples = []
+            self.counters = []
         self.size = size
         super(TransformerAccumulator, self).__init__(**kwargs)
 
-    def handle_sample(self, sample):
+    def handle_sample(self, context, counter, source):
         if self.size >= 1:
-            self.samples.append(sample)
+            self.counters.append(counter)
         else:
-            return sample
+            return counter
 
-    def flush(self):
-        if len(self.samples) >= self.size:
-            x = self.samples
-            self.samples = []
+    def flush(self, context, source):
+        if len(self.counters) >= self.size:
+            x = self.counters
+            self.counters = []
             return x
         return []
