@@ -21,18 +21,23 @@ then
     echo "notification_driver=cinder.openstack.common.notifier.rabbit_notifier" >> $CINDER_CONF
 fi
 
-# For compatibility with Grizzly
 QUANTUM_CONF=/etc/quantum/quantum.conf
 if ! grep -q "notification_driver=quantum.openstack.common.notifier.rabbit_notifier" $QUANTUM_CONF
 then
     echo "notification_driver=quantum.openstack.common.notifier.rabbit_notifier" >> $QUANTUM_CONF
 fi
 
-# Havana release onward
-NEUTRON_CONF=/etc/neutron/neutron.conf
-if ! grep -q "notification_driver=neutron.openstack.common.notifier.rabbit_notifier" $NEUTRON_CONF
+# For nova we set both the rabbit notifier and the special ceilometer
+# notifier that forces one more poll to happen before an instance is
+# removed.
+NOVA_CONF=/etc/nova/nova.conf
+if ! grep -q "notification_driver=ceilometer.compute.nova_notifier" $NOVA_CONF
 then
-    echo "notification_driver=neutron.openstack.common.notifier.rabbit_notifier" >> $NEUTRON_CONF
+    echo "notification_driver=ceilometer.compute.nova_notifier" >> $NOVA_CONF
+fi
+if ! grep -q "notification_driver=nova.openstack.common.notifier.rabbit_notifier" $NOVA_CONF
+then
+    echo "notification_driver=nova.openstack.common.notifier.rabbit_notifier" >> $NOVA_CONF
 fi
 
 # SPECIAL CASE
